import java.io.*;
import java.math.BigInteger;
import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Function;

public class TestGen {
    Random random = new Random(0xB000B);

    int testCount;
    public static final int MAX_N = 1000_00;
    public static final int MAX_VAL = 1_000_000_000;

    void print(int[] x, int[] d) throws IOException {
        if (x.length > MAX_N)
            throw new IllegalArgumentException("Too long array");

        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println("Generating test #" + testCount);
        out.println(x.length);
        for (int i = 0; i < x.length; ++i)
            out.println(x[i] + " " + d[i]);
        out.close();
    }


    void genSamples() throws IOException {
        print(new int[]{1, 3}, new int[]{2, 2});
        print(new int[]{2, 6, 8}, new int[]{2, 2, 1});
        String s = "dashasoska";
        int[] x = new int[s.length() / 2];
        int[] d = new int[s.length() / 2];
        for (int i = 0; i < s.length(); i += 2) {
            x[i / 2] = s.charAt(i);
            d[i / 2] = s.charAt(i + 1);
        }
        print(x, d);
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();
        genHandleTests();
        for (int i = 0; i < 10; ++i)
            genYES(MAX_N);
        for (int i = 0; i < 5; ++i)
            genRandom(MAX_N);
        genWithZero(MAX_N);
        genAllZeros(MAX_N);
        genAllZeros(MAX_N);
        for (int i = 0; i < 5; ++i)
            genNO(MAX_N);
    }

    private void genNO(int n) throws IOException {
        int[] x = new int[n];
        int[] d = new int[n];
        int k = (int)(n * random.nextDouble());
        genArray(x, d, n - k);
        int mn = Integer.MAX_VALUE;
        int mx = Integer.MIN_VALUE;
        for (int i = 0; i < n - k; ++i) {
            mn = Math.min(x[i] + d[i], mn);
            mx = Math.max(x[i] - d[i], mx);
        }
        for (int i = n - k; i < n; ++i)
            if (random.nextBoolean()) {
                x[i] = mn + random.nextInt(MAX_VAL - mn);
                d[i] = random.nextInt(Math.min(MAX_VAL - x[i], x[i] - mn));
            } else {
                x[i] = mx - random.nextInt(mx);
                d[i] = random.nextInt(Math.min(mx - x[i], x[i]));
            }
        print(x, d);
    }

    private void genAllZeros(int n) throws IOException {
        int X = random.nextInt(MAX_VAL);
        int[] x = new int[n];
        int[] d = new int[n];
        for (int i = 0; i < n; ++i) {
            x[i] = X;
            d[i] = 0;
        }
        print(x, d);
    }

    private void genWithZero(int n) throws IOException{
        int[] x = new int[n];
        int[] d = new int[n];
        int X = genArray(x, d, n - 1);
        x[n - 1] = X;
        d[n - 1] = 0;
        print(x, d);
    }


    private void genRandom(int n) throws IOException{
        int[] x = new int[n];
        int[] d = new int[n];
        for (int i = 0; i < n; ++i) {
            x[i] = random.nextInt(MAX_VAL + 1);
            d[i] = random.nextInt(MAX_VAL + 1);
        }
        print(x, d);
    }

    private void genYES(int n) throws IOException{
        int[] x = new int[n];
        int[] d = new int[n];
        genArray(x, d, n);
        print(x, d);
    }

    private int genArray(int[] x, int[] d, int n) {
        int X = random.nextInt(MAX_VAL);
        for (int i = 0; i < n; ++i) {
            x[i] = random.nextInt(MAX_VAL);
            int minD = Math.abs(X - x[i]);
            d[i] = random.nextInt(MAX_VAL - minD) + minD;
        }
        return X;
    }

    private void genHandleTests() throws IOException {
        print(new int[] {MAX_VAL}, new int[]{MAX_VAL});
        print(new int[] {0}, new int[]{0});
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
