/**
 * Created by Grigorii Shovkoplias on 06.11.2015.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        done = 0;
        genSamples();
        genRand(3, 10, 10);
        genEmpty(10, 10, 3);
        genBackspaces(2, 16, 5);
        genBackspaces2(1, 20, 10);
        genNext(10, 20, 10);
        genCP(2, 13, 20);

        int a[] = new int[]{10, 50, 100, 200, 500};
        for (int i = 0; i < a.length; i++) {
            genRand(rand(MAXN) + 1, a[i], rand(MAXK) + 1);
            genBackspaces2(rand(MAXN) + 1, a[i], rand(MAXK) + 1);
            genNext(gand(MAXN), a[i], rand(MAXK) + 1);
            genCP(rand(MAXN) + 1, a[i], rand(MAXK) + 1);
        }

        genRand(MAXN, MAXM, MAXK);
        genEmpty(MAXN, MAXM, MAXK);
        genBackspaces(MAXN, MAXM, MAXK);
        genBackspaces2(MAXN, MAXM, MAXK);
        genNext(MAXN, MAXM, MAXK);
        genCP(MAXN, MAXM, MAXK);


    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN = 10;
    private final int MAXM = 1000;
    private final int MAXK = 1000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private int gand(int n) {
        return rand(n / 2) + n / 2 + 1;
    }

    private char randChar() {
        return (char) (rand(26) + 'a');
    }

    private char zeroOrOne() {
        return (char) ('0' + rnd.nextInt(2));
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private void print(int n, int k, String[] test) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(n + " " + test.length + " " + k);
        for (int i = 0; i < test.length; i++) {
            out.println(test[i]);
        }
        out.close();
    }

    //begin generators

    final String next = "Next";
    final String copy = "Copy";
    final String backspace = "Backspace";
    final String paste = "Paste";
    final String[] commands = new String[]{next, copy, paste, backspace};

    private void genSamples() throws IOException {
        print(2, 10, new String[]{next, "a", "b", copy, next, paste, paste});
        print(10, 10, new String[]{"a", "b", copy, backspace, paste, paste, backspace});
        print(1, 3, new String[]{"a", copy, paste, copy, paste, copy, paste});
        print(3, 10, new String[]{"a", "b", next, "c", next});
    }

    private void genRand(int n, int m, int k) throws IOException {
        String[] test = new String[m];
        for (int i = 0; i < m; i++) {
            int tmp = rand(5);
            if (tmp == 4) {
                test[i] = String.valueOf(randChar());
            } else {
                test[i] = commands[tmp];
            }
        }
        print(n, k, test);
    }

    private void genBackspaces(int n, int m, int k) throws IOException {
        String[] test = new String[m];
        for (int i = 0; i < 4 * m / 5; i++) {
            int tmp = rand(5);
            if (tmp == 4) {
                test[i] = String.valueOf(randChar());
            } else {
                test[i] = commands[tmp];
            }
        }
        for (int i = 4 * m / 5; i < m; i++) {
            test[i] = backspace;
        }
        print(n, k, test);
    }

    private void genBackspaces2(int n, int m, int k) throws IOException {
        String[] test = new String[m];
        test[0] = String.valueOf(randChar());
        int i;
        for (i = 1; i < m / 2; i += 2) {
            test[i] = copy;
            test[i + 1] = paste;

        }
        for (; i < m; i++) {
            test[i] = backspace;
        }
        print(n, k, test);
    }

    private void genNext(int n, int m, int k) throws IOException {
        m -= m % 2;
        String[] test = new String[m];
        for (int i = 0; i < m; i += 2) {
            int tmp = rand(5);
            if (tmp == 4) {
                test[i] = String.valueOf(randChar());
            } else {
                test[i] = commands[tmp];
            }
            test[i + 1] = next;
        }
        print(n, k, test);
    }

    private void genEmpty(int n, int m, int k) throws IOException {
        String[] test = new String[m];
        for (int i = 0; i < m; i++) {
            test[i] = next;
        }
        print(n, k, test);
    }

    private void genCP(int n, int m, int k) throws IOException {
        m -= ((m + 1) % 2);
        String[] test = new String[m];
        test[0] = String.valueOf(randChar());
        for (int i = 1; i < m; i += 2) {
            test[i] = copy;
            test[i + 1] = paste;
        }
        print(n, k, test);
    }




    //end generators

}
