import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;

import java.util.ArrayList;

public class Check implements Checker {

    class Fenwick {
        int[] s;
        Fenwick(int n) {
            s = new int[n];
        }

        void upd(int v, int val) {
            for (; v < s.length; v |= v + 1)
                s[v] += val;
        }

        int get(int l, int r) {
            return getPrefix(r) - getPrefix(l - 1);
        }

        int getPrefix(int v) {
            int ret = 0;
            for (; v >= 0; v = (v & (v + 1)) - 1)
                ret += s[v];
            return ret;
        }
    }

    boolean ensureLimits(int n, int from, int to) {
        return from <= n && n <= to;
    }

    ArrayList<Integer>[] rights;

    @Override
    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int cAns = ouf.nextInt();
        int jAns = ans.nextInt();
        if (cAns != jAns) {
            return new Outcome(Outcome.Type.WA, String.format("expected: %d, found: %d", jAns, cAns));
        }

        int n = inf.nextInt(), m = inf.nextInt();
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) {
            a[i] = ouf.nextInt();
            if (!ensureLimits(a[i], 1, cAns))
                return new Outcome(Outcome.Type.WA, String.format("Bad a[%d]", i));
            --a[i];
        }

        rights = new ArrayList[n];
        for (int i = 0; i < n; ++i)
            rights[i] = new ArrayList<>();

        int[] l = new int[m];
        int[] r = new int[m];
        for (int i = 0; i < m; ++i) {
            l[i] = inf.nextInt();
            r[i] = inf.nextInt();
            --l[i];
            --r[i];
            rights[l[i]].add(r[i]);
        }
        int[] prev = new int[cAns];
        for (int i = 0; i < prev.length; ++i)
            prev[i] = -1;

        Fenwick fenwick = new Fenwick(n);
        for (int i = n - 1; i >= 0; --i) {
            if (prev[a[i]] != -1)
                fenwick.upd(prev[a[i]], -1);
            fenwick.upd(i, 1);
            prev[a[i]] = i;
            for (int j = 0; j < rights[i].size(); ++j) {
                int diff = fenwick.get(i, rights[i].get(j));
                if (diff != rights[i].get(j) - i + 1)
                    return new Outcome(Outcome.Type.WA, String.format("On segment [%d;%d]", i + 1, rights[i].get(j) + 1));
            }
        }
        return new Outcome(Outcome.Type.OK, String.format("%d", jAns));
    }
}
